/*file*********** MELNK_DRV : ***********************************************/
/*                                                                          */
/*                                                                          */
/*        JL080DEF.H : Common header of JL-080 access driver                */
/*                                                                          */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/*       [Defines]                                                          */
/*          Definetion of Host Interface registers                          */
/*                                                                          */
/*                                                                          */
/******** Copyright(C) 2002 Yaskawa Electric Corporation ********************/
/*                                                                          */
/*      Ver 1.00    2002.03.01   Initial coding    I.Matsumura              */
/*      Ver 1.11    2003.02.25                     I.Matsumura              */
/*              - add error code                                            */
/*      Ver 1.30    2003.11.14                     I.Matsumura              */
/*              - add error code                                            */
/*              - add new member to CHANNEL_INFO structure                  */
/*                                                                          */
/****************************************************************************/
#ifndef		YENETDEF_H_
#define		YENETDEF_H_

/***********************************************************************/
/*   Type macro                                                        */
/***********************************************************************/
#ifndef         SHORT
typedef		short          SHORT;
#endif       /* SHORT */
#ifndef         USHORT
typedef		unsigned short USHORT;
#endif       /* USHORT */
#ifndef         LONG
typedef		long           LONG;
#endif       /* LONG */
#ifndef         ULONG
typedef		unsigned long  ULONG;
#endif       /* ULONG */
#ifndef         VOID
typedef		void           VOID;
#endif       /* VOID */
#ifndef         INT
typedef		int            INT;
#endif       /* int */
#ifndef         UINT
typedef		unsigned int   UINT;
#endif       /* UINT */
#ifndef         UCHAR
typedef		unsigned char   UCHAR;
#endif       /* UCHAR */

#ifndef		NULL
#define 	NULL	((void *)0)
#endif       /* NULL */

#ifndef		TRUE
#define		TRUE	1
#endif		/* TRUE */
#ifndef		FALSE
#define		FALSE	0
#endif		/* FALSE */

/************************************************************************/
/*      HOST I/F register bit pattern definition                        */
/************************************************************************/

/* CMD regs */
#define	YN_CMD_C1MTX_R		0x0001
#define YN_CMD_C2MTX_R		0x0002
#define YN_CMD_PAR_R		0x0004
#define YN_CMD_C1MRX_R		0x0008
#define YN_CMD_C2MRX_R		0x0010
#define YN_CMD_WD_R			0x0020
#define YN_CMD_LTX_RA		0x0040
#define YN_CMD_LTX_RB		0x0080
#define YN_CMD_ENA_CER		0x0100
#define YN_CMD_P_RSTRQ		0x0400
#define YN_CMD_C1ABT_R		0x0800
#define YN_CMD_C2ABT_R		0x1000
#define YN_CMD_RST_CER		0x8000

/* STS regs */
#define YN_STS_LTX_UA		0x0040
#define YN_STS_LTX_UB		0x0080
#define YN_STS_RDY			0x2000
#define YN_STS_ERRL			0x4000
#define YN_STS_ERR			0x8000

/* intS, IMK regs */
#define YN_INTS_C1MTX_I		0x0001
#define YN_INTS_C2MTX_I		0x0002
#define YN_INTS_PAR_I		0x0004
#define YN_INTS_C1MRX_I		0x0008
#define YN_INTS_C2MRX_I		0x0010
#define YN_INTS_TXEN_I		0x0020
#define YN_INTS_TXST_I		0x0040
#define YN_INTS_P_RSTAK		0x0400
#define	YN_INTS_TIM1		0x1000
#define YN_INTS_TIM1D		0x2000
#define YN_INTS_SFR			0x8000

/* ESTS regs */
#define YN_ESTS_CRC_ERR		0x0001
#define YN_ESTS_ABT_ERR		0x0002
#define YN_ESTS_PAR_ERR		0x0004
#define YN_ESTS_TMCYCOVR	0x0008
#define YN_ESTS_ALG_ERR		0x0010
#define YN_ESTS_SPT_ERR		0x0020
#define YN_ESTS_TGLCOVR		0x0200
#define YN_ESTS_TGL_ERR		0x0400
#define YN_ESTS_TUR_ERR		0x0800
#define YN_ESTS_RFO_ERR		0x1000
#define YN_ESTS_FE_RSM		0x2000
#define YN_ESTS_EWDTOVR		0x4000
#define YN_ESTS_WDTOVR		0x8000

#define YN_ESTS_APPL_ERR	(YN_ESTS_TGL_ERR | YN_ESTS_TMCYCOVR | YN_ESTS_PAR_ERR)
#define YN_ESTS_SYS_ERR		(YN_ESTS_WDTOVR | YN_ESTS_EWDTOVR | YN_ESTS_RFO_ERR | YN_ESTS_TUR_ERR)

/* MOD regs */
#define	YN_MOD_PROCEL_1NS	0x0000
#define	YN_MOD_PROCEL_2NS	0x0001

#define YN_MOD_SPEED_2M		0x0018
#define YN_MOD_SPEED_4M		0x0010
#define YN_MOD_SPEED_10M	0x0008
#define YN_MOD_SPEED_20M	0x0000

#define YN_MOD_CODE_MAN		0x00c0
#define YN_MOD_CODE_NRZI	0x0040

#define	YN_MOD_BMOD_STATIC	0x0000
#define	YN_MOD_BMOD_ALT		0x0100
#define	YN_MOD_BMOD_RALT	0x0300

#define	YN_MOD_MTYP_C1		0x0000
#define	YN_MOD_MTYP_C2		0x0400

#define	YN_MOD_INTLV_PLS	0x0800
#define	YN_MOD_INTLV_LVL	0x0000

#define	YN_MOD_ESYNCS		0x1000
#define	YN_MOD_LBTST		0x2000
#define	YN_MOD_TYPE_MASTER	0x4000
#define	YN_MOD_TYPE_SLAVE	0x0000
#define	YN_MOD_INT_FRS		0x8000
#define	YN_MOD_MASK			0x0300	/* BMOD mask */

/* TMBC,RMBC */
#define YN_MBC_WD		0x0002

/* default parameters */
#define	DEF_IMK_REG			0		/* IMK register */
#define	DEF_RAM_SIZE		2048	/* RAM word size */
/* 1NS */
#define	DEF_1NS_MA			0		/* My address */
#define	DEF_1NS_MA_MAX		14		/* Max. number of slaves */
#define	DEF_1NS_T_MCYC		2000	/* Tmcyc */
#define	DEF_1NS_T_CYC		130		/* Tcyc */
#define	DEF_1NS_BYTE		17		/* Length of Cyclick trans. data */
#define	DEF_1NS_DEV			0		/* Devision */
#define	DEF_1NS_PREL		1		/* Length of Preamble */
#define	DEF_1NS_INT1_DLY	8		/* PV of short1 delay timer */
#define	DEF_1NS_RAM_SPD		0		/* External RAM speed  */
#define	DEF_1NS_ROM_SPD		0		/* External ROM speed  */
#define	DEF_1NS_LBSS_REG	0x300	/* LBSS register */
#define	DEF_1NS_RMBD_REG	0x780	/* RMBD register */
/* 2NS */
#define	DEF_2NS_MA			1		/* My address */
#define	DEF_2NS_MA_MAX		14		/* Max. number of slaves */
#define	DEF_2NS_T_MCYC		4000	/* Tmcy*/
#define	DEF_2NS_T_CYC		360		/* Tcyc */
#define	DEF_2NS_BYTE		30		/* Length of Cyclick trans. data */
#define	DEF_2NS_DEV			0		/* Devision */
#define	DEF_2NS_PREL		1		/* Length of Preamble */
#define	DEF_2NS_INT1_DLY	32		/* PV of short1 delay timer ~4 */
#define	DEF_2NS_RAM_SPD		0		/* External RAM speed  */
#define	DEF_2NS_ROM_SPD		0		/* External ROM speed  */
#define	DEF_2NS_LBSS		0x300	 /* Pointer to alt. buff. B (LBSS) */
#define	DEF_2NS_MAX_RTRY	1		/* Max. num. of retry */
#define	DEF_2NS_C2M_CH		1		/* C2-Master flag */
#define	DEF_2NS_MSG_PTR		0x632	/* Pointer to messege buffer */
#define	DEF_2NS_C1MSG_SZ	128		/* C1-message size */
#define	DEF_2NS_C2MSG_SZ	128		/* C2-message size */
#define	DEF_2NS_WDT			15		/* PV of Watch dog timer */
#define	DEF_2NS_C2DLY		180		/* C2-Master send delay */

/* Error Code */
#ifndef OK
#define	OK				0
#endif

#define	ERROR_INIT_CHANNEL			(-1)
#define	ERROR_INVALID_PROTOCOL		(-2)		/* yns_set_regs() */
#define	ERROR_INVALID_BMODE			(-3)		/* yns_set_regs() */
#define	ERROR_INVALID_PARAMETER		(-4)		/* yns_set_param() */

#define	ERROR_NOT_READY				(-5)		/* yns_reset() */
#define	ERROR_POLL_TIMEOUT_REG		(-6)		/* yns_sequent_init() */
#define	ERROR_POLL_TIMEOUT_PAR		(-7)		/* yns_sequent_init() */
#define	ERROR_POLL_TIMEOUT_ENA		(-8)		/* yns_sequent_init() */
#define ERROR_RAM_SIZE_OVER			(-9)

#define ERROR_STS_ERR				(-10)
#define ERROR_RAM_CHECK				(-11)
#define ERROR_SLOTCH_OVER			(-12)

#define	ERROR_INVALID_ST_NO			(-20)
#define	ERROR_ALT_BUFFER			(-21)
#define ERROR_TOGGLE_BUFFER			(-22)
#define	ERROR_REQ_SIZE				(-23)
#define	ERROR_RECV_DATA				(-24)		/* yns_read_ldata() */
#define ERROR_SYNC_NO_RECV			(-25)
#define ERROR_NO_SYNC				(-26)
#define ERROR_JL080_SYSTEM			(-27)
#define ERROR_JL080_APPL			(-28)

#define	ERROR_STS_MSG				(-30)
#define	ERROR_RCV_ABORT_MSG			(-31)
#define	ERROR_HNDSHAKE_MSG			(-32)
#define	ERROR_SNDBUF_UNDERFLOW		(-33)
#define	ERROR_RCVBUF_OVERFLOW		(-34)
#define	ERROR_NOT_ENOUGH_MSGAREA	(-35)
#define	ERROR_RECV_BUSY				(-36)

#define YENET_INIT_CHBUF			0x1020	/*  */
#define YENET_INIT_DONE				0x2040	/*  */

#define YN_C1_MASTER				0	/* C1 master */
#define YN_C2_MASTER				1	/* C21 master */

#define YN_CTRL_CDRW				0x03	/* CDRW code */
#define YN_CTRL_MDS					0x04	/* MDS code */
#define YN_CTRL_CDRW_ACK			0x01	/* ACK code */
#define YN_CTRL_MDS_ACK				0x90	/* MDS ACK code */

#define YN_CTRL_ERRCODE				0x00FF	/* Receve Error code */

/***********************************************************************/
/*	Structure of HOST I/F registers                                    */
/***********************************************************************/
typedef struct _YENET_HOST_IF_REGS {
	volatile USHORT cmd;	/* Command to CERF */
	volatile USHORT sts;	/* Status */
	volatile USHORT ints;	/* Interrupt Status */
	volatile USHORT imk;	/* Interrupt Mask, same as ints register */
	volatile USHORT ests;	/* Error Status */
	volatile USHORT mod;	/* Mode */
	volatile USHORT lbs;	/* Link Buffer Select */
	volatile USHORT lbp;	/* Link Buffer Access Pointer */
	volatile USHORT lbd;	/* Link Buffer Data Port */
	volatile USHORT ubp;	/* Universal Buffer Access Pointer */
	volatile USHORT ubd;	/* Universal Buffer Data Port */
	volatile USHORT tmbc;	/* Transmit Message Buffer Control */
	volatile USHORT tmbd;	/* Transmit Message Buffer Data Port */
	volatile USHORT rmbc;	/* Receive Message Buffer Control */
	volatile USHORT rmbd;	/* Receive Message Buffer Data Port */
	volatile USHORT mctr;	/* Message Conter */
} YENET_HOST_IF_REGS;


/***********************************************************************/
/*    Communication Parameter                                          */
/***********************************************************************/
typedef struct _YENET_COM_PAR {
	USHORT ma;				/* My Address */
	USHORT ma_max;			/* Max. Number of Slaves */
	USHORT t_mcyc;			/* Tmcyc */
	USHORT t_cyc;			/* Tcyc */
	USHORT byte;			/* Transimit Data Size (Bytes) */
	USHORT dev;				/* Devision */
	USHORT prel;			/* Preamble Length */
	USHORT int1_dly;		/* int1 Delay */
	USHORT ram_spd;			/* External RAM Speed */
	USHORT rom_spd;			/* External ROM Speed */
	USHORT lbss;			/* Alt buf.B start Adrress*/
	USHORT max_rtry;		/* Max Number of Retry */
	USHORT c2m_ch;			/* Time slot */
	USHORT msg_ptr;			/* Message buf. pointer */
	USHORT c1msg_sz;		/* C1 message size */
	USHORT c2msg_sz;		/* C2 message size */
	USHORT wdt;				/* Watch dog setting */
	USHORT c2dly;			/* Transfer message delay for C2 */
} YENET_COM_PAR;


typedef struct _YENET_USER_PAR {
	USHORT mod;				/* MOD register */
	USHORT ma;				/* My Address */
	USHORT ma_max;			/* Max. Number of Slaves */
	USHORT t_mcyc;			/* Tmcyc */
	USHORT t_cyc;			/* Tcyc */
	USHORT byte;			/* Transimit Data Size (Bytes) */
	USHORT dev;				/* Devision */
	USHORT max_rtry;		/* Max Number of Retry */
	USHORT c2m_ch;			/* Time slot */
	USHORT wdt;				/* Watch dog setting */
} YENET_USER_PAR;


/***********************************************************************/
/*    Register Parameter                                               */
/***********************************************************************/
typedef struct _YENET_REG_PAR {
	USHORT mod;				/* MOD register */
	USHORT imk;				/* IMK register */
	USHORT rmbd;			/* RMBD register */
	USHORT mctr;			/* MCTR register */
} YENET_REG_PAR;

typedef struct _CHANNEL_INFO {
	YENET_HOST_IF_REGS *hif_reg_top;	/* Host I/F register top save area */
	USHORT ma_max;						/* Max. nods save area */
	USHORT ma;							/* My address save area */
	USHORT link_sz;						/* Buffer size */
	USHORT buff_mod;					/* Burffer mode (FIX or ALT or MIX) */
	USHORT alt_buff;					/* Current buffer (A or B) */
	USHORT ubp_data;					/* UBP backup data */
	USHORT link_sadr[2];				/* Link buffer offset address */
	USHORT link_radr[2];				/* Link buffer offset address */
	USHORT pars_data;					/* Communication Param. address */
	USHORT c1msg_sz;					/* C1 message buffer size */
	USHORT c2msg_sz;					/* C2 message buffer size */
	USHORT c1msg_adr;					/* C1 message buffer address */
	USHORT c2msg_adr;					/* C2 message buffer address */
	USHORT msgpkt_sz;					/* Packet size of message */
	USHORT check_code;
	USHORT net_clk;						/* Network time */
} CHANNEL_INFO;


#endif /* _YENETDEF_H */
