/*file*********** MELNK_DRV : ***********************************************/
/*                                                                          */
/*                                                                          */
/*                JL080LNK.C : JL-080 interface access driver               */
/*                                                                          */
/*                                                                          */
/******** Copyright(C) 2002 Yaskawa Electric Corporation ********************/
/*                                                                          */
/*      Ver 1.00    2002.03.01   Initial coding    I.Matsumura              */
/*      Ver 1.10    2002.06.01                     I.Matsumura              */
/*                - add function : write network time - yns_set_netclk()    */
/*                               : read network time - yns_get_netclk()     */
/*      Ver 1.20    2003.04.11                     I.Matsumura              */
/*                - remove functions : yns_read_lpart(),yns_write_lpart()   */
/*                - add functions : yns_read_ldata2(),yns_write_ldata2()    */
/*      Ver 1.30    2003.11.20                     I.Matsumura              */
/*                <1> add a new function : yns_get_gblstat()                */
/*                <2> fix return code(ESTS) of yns_read_comstat()           */
/*                <3> fix calculation of ubp reg. data                      */
/*                <4> delete the station number check                       */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/*       [Functions]                                                        */
/*      yns_read_ldata()              Read link data                        */
/*      yns_write_ldata()             Write link data                       */
/*      yns_read_ldata2()             Read link data with a packet header   */
/*      yns_write_ldata2()            Write link data with a packet header  */
/*      yns_begin_link_acc()	      Begin link read/write access          */
/*      yns_end_link_acc()	          Finish link read/write access         */
/*      yns_link_exchange()           Read and Write cyclic trans. data     */
/*      yns_read_comstat()            Read communication status             */
/*      yns_set_netclk()              Set network clock                     */
/*      yns_get_netclk()              Get network clock                     */
/*                                                                          */
/*                                                                          */
/******** Copyright(C) 2002 Yaskawa Electric Corporation ********************/

#ifndef YENET_LOCAL_DEFINES_
#define YENET_LOCAL_DEFINES_
#endif /*_YENET_LOCAL_DEFINES */

/* includes */
#include "jl080gbl.h"

/* loacal defines */
#define	POLL_TIMES20	20			/* Polling timer 20count */
#define CLR_CTRL_CODE	YN_CTRL_ERRCODE		/* Default CTRL code */
#define ESTS_MASK		0xDE0C


/* forward declarations */
#ifdef __cplusplus
extern	"C"	{
#endif
int yns_link_exchange( CHANNEL_INFO *chbuffp, USHORT *rbuffp,
						 USHORT *sbuffp, UINT size, USHORT *ests);
int yns_read_ldata( CHANNEL_INFO *chbuffp, 
					UINT st_no, USHORT *rbuffp, UINT size );
int yns_write_ldata( CHANNEL_INFO *chbuffp, 
					UINT st_no, USHORT *sbuffp, UINT size );
int yns_read_ldata2( CHANNEL_INFO *chbuffp, 
					UINT st_no, USHORT *sts, USHORT *rbuffp, UINT size );
int yns_write_ldata2( CHANNEL_INFO *chbuffp,
					UINT st_no, USHORT sts, USHORT *sbuffp, UINT size );
int yns_begin_link_acc(CHANNEL_INFO *);
int yns_end_link_acc(CHANNEL_INFO *);
USHORT yns_read_comstat( CHANNEL_INFO *pchannel);
USHORT yns_set_netclk( CHANNEL_INFO *chbuffp, UINT cycle );
USHORT yns_get_netclk( CHANNEL_INFO *chbuffp );
int yns_get_gblstat( CHANNEL_INFO *chbuffp, USHORT *t_mcyc);

#ifdef __cplusplus
}
#endif


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*       yns_link_exchange()              Read and Write cyclic trans. data */
/*                                                                          */
/*Function :                                                                */
/*      Read cyclic trans. data in 1NS/2NS.                                 */
/*                                                                          */
/*Return code :                                                             */
/*      OK :                              Normal End                        */
/*      n < 0 :                           error code                        */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :           Channel buffer address            */
/*      USHORT *rbuffp :                  Read Buffer                       */
/*      USHORT *sbuffp :                  Write Buffer                      */
/*      UINT size    :                    Buffer size                       */
/*      USHORT *status :                  Receive error status              */
/*                                                                          */
/*Note :                                                                    */
/*     C1 master only use this function.                                    */
/*                                                                          */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
int yns_link_exchange( CHANNEL_INFO *chbuffp, USHORT *rbuffp,
						 USHORT *sbuffp, UINT size, USHORT *ests)
{
	int		ch;			/* Station number */
	int		ret, rc;	/* return code */
	USHORT	*bufp;		/* buffer pointer */

/* Begin link access */
	if((ret = yns_begin_link_acc(chbuffp)) != OK)
	{
		yns_end_link_acc(chbuffp);
		return ret;
	}

/* Read/Write cyclic data */
	for(ch = 1; ch <= chbuffp->ma_max; ch++)
	{
	/* Write cyclic trans. data */
		bufp = sbuffp + size*(ch-1);
		if((rc = yns_write_ldata(chbuffp, ch, bufp, size)) < 0)
		{
			ret = rc;
		}

	/* Read cyclic trans. data */
		bufp = rbuffp + size*(ch-1);
		if((rc = yns_read_ldata(chbuffp, ch, bufp, size)) < 0)
		{
			ret = rc;
		}
	}

/* End link access */
	rc = yns_end_link_acc(chbuffp);

/* Check error status */
	if((*ests = yns_read_comstat(chbuffp)) != 0)
	{
		ret = ERROR_STS_ERR;
	}

	if(ret != OK) return ret;

	return rc;
}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*       yns_read_ldata()               Read cyclic trans. data or link data*/
/*                                                                          */
/*Function :                                                                */
/*      Read cyclic trans. data in 1NS/2NS.                                 */
/*                                                                          */
/*Return code :                                                             */
/*      short :                           n > 0 : size of read data (word)  */
/*                                        n = 0 : no data to be read        */
/*                                        n < 0 : error code                */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :           Channel buffer address            */
/*      UINT st_no :                      Station number to be read         */
/*      USHORT *rbuffp :                  Buffer                            */
/*      UINT size :                       Buffer size                       */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*            1.30<3> fix calculation of ubp reg. data                      */
/*            1.30<4> delete the station number check                       */
/*                                                                          */
/****************************************************************************/
int yns_read_ldata( CHANNEL_INFO *chbuffp, 
					UINT st_no, USHORT *rbuffp, UINT size)
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	int		ret;			/* Return code */
	USHORT	ubp_data;		/* Offset address */
	USHORT	sts;			/* Status check code */
	USHORT	*prd;			/* read buffer pointer */

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

/* Read data */
	if(size > (UINT)(chbuffp->link_sz - 1))
		size = chbuffp->link_sz - 1;
	ret = size;

/* Get receive buffer offset */
	ubp_data = chbuffp->link_radr[chbuffp->alt_buff];

/* Add offset of station number */
	ubp_data += (USHORT)(chbuffp->link_sz * st_no);

/* Set offset address to UBP.reg */
	ynb_set_ubp( hirp, ubp_data );

/* Save receive status */
	sts = ynb_get_ubd( hirp );

/* Check receive status, when data is from slave. */
	if (sts == CLR_CTRL_CODE)
	{
		return ERROR_RECV_DATA;	/* Receive error */
	}

	prd = rbuffp;			/* set buffer pointer */
/* Read from UBD */
	while ( (size--) > 0 )
	{
		*prd = ynb_get_ubd( hirp );
    	++prd;
	}

	ynb_set_ubp( hirp, ubp_data );
	ynb_set_ubd( hirp, CLR_CTRL_CODE );	/* Set CTRL code */

	return ret;
}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*       yns_write_ldata()             Write cyclic trans. data or link data*/
/*                                                                          */
/*Function :                                                                */
/*      Write cyclic trans. data in 1NS/2NS.                                */
/*                                                                          */
/*Return code :                                                             */
/*      short :                         n >= 0 : size of written data (word)*/
/*                                      n < 0 : error code                  */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :         Channel buffer address              */
/*      UINT st_no :                    Station number                      */
/*      USHORT *sbuffp :                Buffer                              */
/*      UINT size :                     Buffer size                         */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*            1.30<3> fix calculation of ubp reg. data                      */
/*            1.30<4> delete the station number check                       */
/*                                                                          */
/****************************************************************************/
int yns_write_ldata( CHANNEL_INFO *chbuffp, 
					UINT st_no, USHORT *sbuffp, UINT size )
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT 	ubp_data;		/* Offset address */
	USHORT	*pwt;			/* write buffer pointer */
	int 	ret;			/* Return code */

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

/* Read data */
	if(size > (UINT)(chbuffp->link_sz -1))
		size = chbuffp->link_sz -1;
	ret = size;

/* Get receive buffer offset */
	ubp_data = 0;
	if (chbuffp->buff_mod == YN_MOD_BMOD_ALT)
	{
		ubp_data = chbuffp->link_sadr[chbuffp->alt_buff];
	}

/* Add offset of station number */
	ubp_data += (USHORT)(chbuffp->link_sz * st_no);

/* Set offset address to UBP.reg */
	ynb_set_ubp( hirp, ubp_data + 1);

/* Write UBD data */
	pwt = sbuffp;
	while((size--) > 0)
	{
		ynb_set_ubd( hirp, *pwt );
        pwt++;
	}

	return ret;
}

/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*       yns_read_ldata2()              Read cyclic trans. data or link data*/
/*                                                                          */
/*Function :                                                                */
/*      Read cyclic trans. data in 1NS/2NS.                                 */
/*                                                                          */
/*Return code :                                                             */
/*      int :                           n > 0 : size of read data (word)    */
/*                                      n = 0 : no data to be read          */
/*                                      n < 0 : error code                  */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :         Channel buffer address              */
/*      UINT st_no :                    Station number to be read           */
/*      USHORT *rctrl :                 Received St. Address and CTRL code  */
/*      USHORT *rbuffp :                Pointer of Receive buffer           */
/*      UINT size :                     Buffer size                         */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*            1.30<3> fix calculation of ubp reg. data                      */
/*            1.30<4> delete the station number check                       */
/*                                                                          */
/****************************************************************************/
int yns_read_ldata2( CHANNEL_INFO *chbuffp, UINT st_no, 
					USHORT *rctrl, USHORT *rbuffp, UINT size)
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	int		ret;			/* Return code */
	USHORT	ubp_data;		/* Offset address */
	USHORT	*prd;			/* read buffer pointer */

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

/* Read data */
	if(size > (UINT)(chbuffp->link_sz - 1))
	{
		size = chbuffp->link_sz - 1;
	}
	ret = size;

/* Get receive buffer offset */
	ubp_data = chbuffp->link_radr[chbuffp->alt_buff];

/* Add offset of station number */
	ubp_data += (USHORT)(chbuffp->link_sz * st_no);

/* Set offset address to UBP.reg */
	ynb_set_ubp( hirp, ubp_data );

/* Save receive status */
	*rctrl = ynb_get_ubd( hirp );

/* Check receive status, when data is from slave. */
	if (*rctrl == CLR_CTRL_CODE)
	{
		return ERROR_RECV_DATA;	/* Receive error */
	}

	prd = rbuffp;			/* set buffer pointer */
/* Read from UBD */
	while ( (size--) > 0 )
	{
		*prd = ynb_get_ubd( hirp );
    	++prd;
	}

	ynb_set_ubp( hirp, ubp_data );
	ynb_set_ubd( hirp, CLR_CTRL_CODE );	/* Set CTRL code */

	return ret;
}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*       yns_write_ldata2()            Write cyclic trans. data or link data*/
/*                                                                          */
/*Function :                                                                */
/*      Write cyclic trans. data in 1NS/2NS.                                */
/*                                                                          */
/*Return code :                                                             */
/*      short :                         n >= 0 : size of written data (word)*/
/*                                      n < 0 : error code                  */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :         Channel buffer address              */
/*      UINT st_no :                    Station number to be write          */
/*      USHORT sctrl :                  St. Address and CTRL code           */
/*      USHORT *sbuffp :                Pointer of Send Buffer              */
/*      UINT size :                     Buffer size                         */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*            1.30<3> fix calculation of ubp reg. data                      */
/*            1.30<4> delete the station number check                       */
/*                                                                          */
/****************************************************************************/
int yns_write_ldata2( CHANNEL_INFO *chbuffp, 
					UINT st_no, USHORT sctrl, USHORT *sbuffp, UINT size )
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT 	ubp_data;		/* Offset address */
	USHORT	*pwt;			/* write buffer pointer */
	int 	ret;			/* Return code */

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

/* Check data size */
	if(size > (UINT)(chbuffp->link_sz -1))
	{
		size = chbuffp->link_sz -1;
	}
	ret = size;

/* Get receive buffer offset */
	ubp_data = 0;
	if (chbuffp->buff_mod == YN_MOD_BMOD_ALT)
	{
		ubp_data = chbuffp->link_sadr[chbuffp->alt_buff];
	}

/* Add offset of station number */
	ubp_data += (USHORT)(chbuffp->link_sz * st_no);

/* Set offset address to UBP.reg */
	ynb_set_ubp( hirp, ubp_data);

/* Set ST# and CTRL code */
	ynb_set_ubd( hirp, sctrl );

/* Write UBD data */
	pwt = sbuffp;
	while((size--) > 0)
	{
		ynb_set_ubd( hirp, *pwt );
        pwt++;
	}

	return ret;
}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_begin_link_acc()      Begin link read/write access              */
/*                                                                          */
/*Function :                                                                */
/*      Begin link read/write access                                        */
/*                                                                          */
/*Reference :                                                               */
/*                                                                          */
/*Return code :                                                             */
/*      OK :                          Normal End                            */
/*      ERROR_ALT_BUFFER :            Toggle buffer error                   */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *pchannel :      Channel buffer address                */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
int yns_begin_link_acc( CHANNEL_INFO *pchannel )
{
	UINT	i;
	USHORT	sts;
	USHORT	ints;
	int		ret = OK;

/* Reset Watch Dog Timer */
	ynb_set_cmd(pchannel->hif_reg_top, YN_CMD_WD_R);
	pchannel->ubp_data = ynb_get_ubp( pchannel->hif_reg_top );

/* Check buffer mode */
	if( pchannel->buff_mod == YN_MOD_BMOD_STATIC )
	{
		pchannel->alt_buff = 0;
		return ret;
	}

/* Communication cycle start interupt	*/
/* Check start complete					*/
	for(i = 0 ; i < POLL_TIMES20 ; i++)
	{
		ints = ynb_get_ints( pchannel->hif_reg_top );
		if((ints & YN_INTS_TXST_I) != 0)
		{
			break;
		}
	}

	if(i == POLL_TIMES20)
	{
	/* Start not complete		*/
		ret = ERROR_ALT_BUFFER;
	}
	else
	{
	/* Get alt. buffer	*/
		sts = ynb_get_sts( pchannel->hif_reg_top );
		pchannel->alt_buff = ((sts & YN_STS_LTX_UA) ? 1 : 0);
	}

	return ret;
}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_end_link_acc()        Finish link read/write access             */
/*                                                                          */
/*Function :                                                                */
/*      Finish link read/write access                                       */
/*                                                                          */
/*Reference :                                                               */
/*                                                                          */
/*Return code :                                                             */
/*      OK :                              Normal End                        */
/*      ERROR_TOGGLE_BUFFER :             Toggle buffer error               */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *pchannel :       Channel buffer address               */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
int yns_end_link_acc( CHANNEL_INFO *pchannel )
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT		regs;
	USHORT		sts;
	int			ret = OK;

	hirp = pchannel->hif_reg_top;

	ynb_set_ubp( hirp, pchannel->ubp_data );	/* Set old UBP */
	ynb_clear_ints( hirp, YN_INTS_TXST_I );	/* Clear TXST_I */

/* Check buffer mode	*/
	if( pchannel->buff_mod != YN_MOD_BMOD_STATIC )
	{
	/* Request change alt. buffer	*/
		sts = ynb_get_sts( hirp );
		regs = ((sts & YN_STS_LTX_UA) ? YN_CMD_LTX_RB : YN_CMD_LTX_RA);
		ynb_set_cmd( hirp, regs );
	}
	else
	{
	/* Request send (for static buffer mode)	*/
		ynb_set_cmd( hirp, YN_CMD_LTX_RA );
	}

	return(ret);
}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_read_comstat()        Read Jl-080 STS regs.                     */
/*                                                                          */
/*Function :                                                                */
/*      This function read STS register. If error is detected, set          */
/*     ESTS reg. value to err parameter.                                    */
/*                                                                          */
/*Reference :                                                               */
/*                                                                          */
/*Return code :                                                             */
/*      OK :                              Normal End                        */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *pchannel :     Channel buffer address                 */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*            1.30<2> fix return code of yns_read_comstat()                 */
/*                                                                          */
/****************************************************************************/
USHORT yns_read_comstat( CHANNEL_INFO *pchannel)
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT			ret = 0;

	hirp = pchannel->hif_reg_top;

/* Check STS regs. */
	ret = ynb_get_ests( hirp );
	ynb_clear_ests( hirp, ret);

	return (ESTS_MASK & ret);
}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*       yns_set_netclk()               Write network time to global data   */
/*                                                                          */
/*Function :                                                                */
/*      Write network time to global data.                                  */
/*                                                                          */
/*Return code :                                                             */
/*      USHORT :                        network time [trans. cycle]         */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :         Channel buffer address              */
/*      UINT cycle :                    access time                         */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*            1.30<3> fix calculation of ubp reg. data                      */
/*                                                                          */
/****************************************************************************/
USHORT yns_set_netclk( CHANNEL_INFO *chbuffp, UINT cycle )
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT	ubp_data;		/* Offset address */
	USHORT	clk;			/* network time */

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

/* Get receive buffer offset */
	ubp_data = 0;
	if (chbuffp->buff_mod  == YN_MOD_BMOD_ALT)
	{
		ubp_data = chbuffp->link_sadr[chbuffp->alt_buff];
	}

/* Set offset address to UBP.reg */
	ynb_set_ubp( hirp, ubp_data + 4 );

/* Write UBD data */
	clk = chbuffp->net_clk + cycle;
	ynb_set_ubd( hirp, clk );

	chbuffp->net_clk = clk;
	return clk;
}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*       yns_get_netclk()               Read network time to global data    */
/*                                                                          */
/*Function :                                                                */
/*      Read network time to global data.                                   */
/*                                                                          */
/*Return code :                                                             */
/*      USHORT :                        network time [trans. cycle]         */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :         Channel buffer address              */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*            1.30<3> fix calculation of ubp reg. data                      */
/*                                                                          */
/****************************************************************************/
USHORT yns_get_netclk( CHANNEL_INFO *chbuffp )
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT	ubp_data;		/* Offset address */

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

/* Get receive buffer offset */
	ubp_data = chbuffp->link_radr[chbuffp->alt_buff];

/* Set offset address to UBP.reg */
	ynb_set_ubp( hirp, ubp_data + 4 );

/* Read UBD data */
	chbuffp->net_clk = ynb_get_ubd( hirp );

	return chbuffp->net_clk;
}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*       yns_get_gblstat()                Get the status of global frame    */
/*                                                                          */
/*Function :                                                                */
/*      Get the status of global frame.                                     */
/*                                                                          */
/*Return code :                                                             */
/*      OK :                              Normal End                        */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :         Channel buffer address              */
/*      t_mcyc                :         transmission cycle time             */
/*                                                                          */
/*Note :                                                                    */
/*      Slave station only                                                  */
/*Log :                                                                     */
/*            1.30<1> add a new function : yns_get_gblstat()                */
/*                                                                          */
/****************************************************************************/
int yns_get_gblstat( CHANNEL_INFO *chbuffp, USHORT *t_mcyc)
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT	ubp_data;		/* Offset address */
	int		rc;
	USHORT	sts;

	*t_mcyc = 0;
/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

/* Get receive buffer offset */
	ubp_data = chbuffp->link_radr[chbuffp->alt_buff];

/* Read Global status */
	sts = ynb_get_ints( hirp );

/* Check Global status */
	if((sts & YN_INTS_SFR) != 0)
	{
		ynb_set_ubp( hirp, ubp_data );
		sts = ynb_get_ubd( hirp );
		if(sts == CLR_CTRL_CODE)	/* invalid inturrput */
		{
			rc = ERROR_NO_SYNC;
		}
		else	/* ok */
		{
			*t_mcyc = ynb_get_ubd( hirp );
			rc = OK;
		}
	}
	else	/* not receive */
	{
		rc = ERROR_SYNC_NO_RECV;
	}

	ynb_clear_ints( hirp, YN_INTS_SFR );	/* clear status */
	ynb_set_ubp( hirp, ubp_data );
	ynb_set_ubp( hirp, CLR_CTRL_CODE );	/* clear status */

	return sts;
}
